@@ -1,4 +1,3 @@
-
 #!/usr/bin/env python
 #
 #  main.py: a shared, automated test suite for Subversion
@@ -199,6 +198,20 @@ def run_command(command, error_expected, binary_mode=0, *varargs):
   """Run COMMAND with VARARGS; return stdout, stderr as lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed."""
 
+  return run_command_stdin(command, error_expected, binary_mode, 
+                           None, *varargs)
+
+# Run any binary, supplying input text, logging the command line
+def run_command_stdin(command, error_expected, binary_mode=0, 
+                      stdin_lines=None, *varargs):
+  """Run COMMAND with VARARGS; input STDIN_LINES (a list of strings
+  which should include newline characters) to program via stdin - this
+  should not be very large, as if the program outputs more than the OS
+  is willing to buffer, this will deadlock, with both Python and
+  COMMAND waiting to write to each other for ever.
+  Return stdout, stderr as lists of lines.
+  If ERROR_EXPECTED is None, any stderr also will be printed."""
+
   args = ''
   for arg in varargs:                   # build the command string
     args = args + ' "' + str(arg) + '"'
@@ -215,11 +228,15 @@ def run_command(command, error_expected, binary_mode=0, *varargs):
   start = time.time()
   infile, outfile, errfile = os.popen3(command + args, mode)
 
+  if stdin_lines:
+    map(infile.write, stdin_lines)
+
+  infile.close()
+
   stdout_lines = outfile.readlines()
   stderr_lines = errfile.readlines()
 
   outfile.close()
-  infile.close()
   errfile.close()
 
   if verbose_mode:
@@ -251,8 +268,7 @@ def run_svn(error_expected, *varargs):
   """Run svn with VARARGS; return stdout, stderr as lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed.  If
   you're just checking that something does/doesn't come out of
-  stdout/stderr, you might want to use actions.run_and_verify_svn() or
-  actions.run_and_verify_svn_error()."""
+  stdout/stderr, you might want to use actions.run_and_verify_svn()."""
   global config_dir
   return run_command(svn_binary, error_expected, 0,
                      *varargs + ('--config-dir', config_dir))
@@ -375,7 +391,7 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 0):
   load_out.close()
   load_err.close()
 
-  dump_re = re.compile(r'^\* Dumped revision (\d+)\.$')
+  dump_re = re.compile(r'^\* Dumped revision (\d+)\.\r?$')
   expect_revision = 0
   for dump_line in dump_lines:
     match = dump_re.match(dump_line)
@@ -387,7 +403,7 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 0):
     print 'ERROR:  dump failed; did not see revision', head_revision
     raise SVNRepositoryCopyFailure
 
-  load_re = re.compile(r'^------- Committed revision (\d+) >>>$')
+  load_re = re.compile(r'^------- Committed revision (\d+) >>>\r?$')
   expect_revision = 1
   for load_line in load_lines:
     match = load_re.match(load_line)
@@ -410,6 +426,16 @@ def set_repos_paths(repo_dir):
     current_repo_url = string.replace(current_repo_url, '\\', '/')
 
 
+def canonize_url(input):
+  "Canonize the url, if the schema is unknown, returns intact input"
+  
+  m = re.match(r"^((file://)|((svn|svn\+ssh|http|https)(://)))", input)
+  if m:
+    schema = m.group(1)
+    return schema + re.sub(r'//*', '/', input[len(schema):])
+  else:
+    return input
+
 ######################################################################
 # Sandbox handling
 
